#include <string.h>
#include "descriptor/descriptor.h"

int main(int argc, char **argv)
{
  ITEM scene, box;
  ITEM tavolo, molla, box1, box2, box3, box4, box5, box6, joker, specchio;
  ITEM cranio, occhio_dx, occhio_sx, mezzotubo, filtro, antigas, maschera;
  ITEM telaio, manico, superfice, pavimento, parete_dx, parete_sx, soffitto;
  ITEM retro, stanza, asta_teca, scabrote_orecchie, scabrote_corno;
  ITEM scabrote_prot, scabrote_zanna_dx, scabrote_zanna_sx, scabrote_testa;
  ITEM teca, scabrote, asse, chiocciola;

  ITEM black_marble, mirror, blueCeramic, redPlastic;
  ITEM yellowRubber, darkGreen, glass, pinkQuartz;
  ITEM blackCeramic, whiteCeramic, darkGlass, brown, avorio;

  if(argc < 2) {
    fprintf(stderr, "Utilizzo : scenizza [tavolo | teca | stanza]\n");
    exit(-1);
  }

  /* initialize */
  set_background(0.0, 0.0, 0.0, TRUE);
  set_ambient_light(0.5, 0.5, 0.5, 1.0);
  create_point_light(-5.132, 27.318, 10.909,
		     1.0, 1.0, 1.0, 1.2, 0.0, "Light");

  /* define attributes */
  black_marble = create_attribute("NeroMarmo");
  set_color(black_marble, 0.0, 0.0, 0.0, 0.5, 0.5);
  set_reflectivity(black_marble, 0.4, 2);

  mirror = create_attribute("Specchio");
  set_color(mirror, 0.0, 0.0, 0.0, 0.0, 0.0);
  set_reflectivity(mirror, 1.0, 128);

  blueCeramic = create_attribute("AzzurroCeramica");
  set_color(blueCeramic, 0.4, 0.71, 1.0, 0.1, 0.8);
  set_reflectivity(blueCeramic, 0.1, 128);

  redPlastic = create_attribute("RossoPlastica");
  set_color(redPlastic, 1.0, 0.22, 0.22, 0.0, 0.896);
  set_reflectivity(redPlastic, 0.230, 32);

  yellowRubber = create_attribute("GialloGomma");
  set_color(yellowRubber, 1.0, 0.72, 0.0, 0.0, 1.0);
  set_reflectivity(yellowRubber, 0.0, 2);

  darkGreen = create_attribute("VerteOpaco");
  set_color(darkGreen, 0.0, 0.53, 0.0, 0.230, 1.0);
  set_reflectivity(darkGreen, 0.0, 64);

  glass = create_attribute("Vetro");
  set_color(glass, 0.52, 0.72, 1.0, 0.5, 0.5);
  set_reflectivity(glass, 0.39, 2);
  set_transparency(glass, 0.9, 0.44, 10.0, 1.01);

  pinkQuartz = create_attribute("RosaQuarzo");
  set_color(pinkQuartz, 1.0, 0.65, 1.0, 0.12, 1.0);
  set_reflectivity(pinkQuartz, 0.630, 16);
  set_transparency(pinkQuartz, 0.6, 0.43, 1, 1.544);

  blackCeramic = create_attribute("NeroCeramica");
  set_color(blackCeramic, 0.0, 0.0, 0.0, 0.1, 0.8);
  set_reflectivity(blackCeramic, 0.2, 128);

  whiteCeramic = create_attribute("BiancoCeramica");
  set_color(whiteCeramic, 1.0, 1.0, 1.0, 0.1, 0.8);
  set_reflectivity(whiteCeramic, 0.2, 128);

  darkGlass = create_attribute("VetroScuro");
  set_color(darkGlass, 0.52, 0.72, 1.0, 0.5, 0.5);
  set_reflectivity(darkGlass, 0.39, 2);

  brown = create_attribute("MarroneScuro");
  set_color(brown, 0.47, 0.34, 0.0, 0.0, 1.0);
  set_reflectivity(brown, 0.0, 2);

  avorio = create_attribute("Avorio");
  set_color(avorio, 1.0, 1.0, 0.75, 0.2, 0.49);
  set_reflectivity(avorio, 0.3, 16);

  pavimento = read_nurbs("stanza_pavimento.db");
  soffitto = read_nurbs("stanza_soffitto.db");
  parete_dx = read_nurbs("stanza_dx.db");
  parete_sx = read_nurbs("stanza_sx.db");
  retro = read_nurbs("stanza_retro.db");

  if(strcmp(argv[1], "tavolo") == 0) {

    /* load surfaces */
    tavolo = read_nurbs("tavolo.db");
    molla = read_nurbs("joker_molla.db");
    box1 = read_nurbs("box_basso.db");
    box2 = read_nurbs("box_fronte.db");
    box3 = read_nurbs("box_retro.db");
    box4 = read_nurbs("box_dx.db");
    box5 = read_nurbs("box_sx.db");
    box6 = read_nurbs("box_coperchio.db");
    cranio = read_nurbs("joker_cranio.db");
    occhio_dx = read_nurbs("joker_occhio_dx.db");
    occhio_sx = read_nurbs("joker_occhio_sx.db");
    mezzotubo = read_nurbs("joker_mezzotubo.db");
    filtro = read_nurbs("joker_filtro.db");
    maschera = read_nurbs("joker_maschera.dbe");
    manico = read_nurbs("specchio_manico.db");
    telaio = read_nurbs("specchio_contorno.db");
    superfice = read_nurbs("specchio_centro.db");

    box = create_list(box1, "Scatola");
    antigas = create_list(maschera, "Antigas");
    specchio = create_list(manico, "Specchio");

    box = insert_in_list(box, box2);
    box = insert_in_list(box, box3);
    box = insert_in_list(box, box4);
    box = insert_in_list(box, box5);
    box = insert_in_list(box, box6);

    antigas = insert_in_list(antigas, filtro);
    antigas = insert_in_list(antigas, mezzotubo);
    antigas = insert_in_list(antigas, occhio_dx);
    antigas = insert_in_list(antigas, occhio_sx);

    joker = create_list(box, "Joker");
    joker = insert_in_list(joker, molla);
    joker = insert_in_list(joker, cranio);
    joker = insert_in_list(joker, antigas);

    specchio = insert_in_list(specchio, telaio);
    specchio = insert_in_list(specchio, superfice);

    scene = create_list(tavolo, "Scene");
    scene = insert_in_list(scene, joker);
    scene = insert_in_list(scene, specchio);

    stanza = create_list(pavimento, "Stanza");
    stanza = insert_in_list(stanza, soffitto);
    stanza = insert_in_list(stanza, parete_dx);
    stanza = insert_in_list(stanza, parete_sx);
    stanza = insert_in_list(stanza, retro);

    scene = insert_in_list(scene, stanza);

    /* put textues and attributes */
    set_scale(tavolo, 1.0 / 0.6, 1.0 / 0.6, 1.0);
    set_projection_texture(tavolo, black_marble, "textures/marble_5_black.hr",
			   2, TRUE, 0.6, 0.8);
    set_scale(tavolo, 0.6, 0.6, 1.0);

    set_scale(stanza, 0.1, 0.1, 0.1);
    set_cube_texture(stanza, whiteCeramic, blackCeramic);
    set_scale(stanza, 1 / 0.1, 1 / 0.1, 1 / 0.1);

    set_scale(tavolo, 1.0 / 0.6, 1.0 / 0.6, 1.0);
    set_projection_texture(antigas, darkGreen, "textures/mil.hr",
			   2, TRUE, 0.6, 0.8);
    set_scale(tavolo, 0.6, 0.6, 1.0);

    set_attribute(occhio_dx, glass);
    set_attribute(occhio_sx, glass);
    set_attribute(molla, yellowRubber);
    set_attribute(box, redPlastic);
    set_attribute(specchio, blueCeramic);
    set_attribute(superfice, mirror);
    set_bump_texture(cranio, pinkQuartz, "textures/techtile.hr",
		     0.2, 0.8, 0.2, 0.8, 1, 0.2, 1.0);

    /* place/scale objects */
    set_translate(tavolo, 2.141, -2.622, 0.991);
    set_scale(tavolo, 1.267, 1.267, 1.152);
    set_scale(tavolo, 1.2, 1.2, 1.0);

    set_z_rotate(joker, 90, 1);
    set_z_rotate(box, -90, 1);
    //set_z_rotate(pavimento, -90, 1);
    set_scale(joker, 0.7, 0.7, 0.7);
    set_scale(box, 1.2, 1.2, 1.0);
    //set_scale(pavimento, 3.0, 3.0, 1.0);
    set_translate(joker, -4.200, 8.199, -4.100);
    set_translate(antigas, 0, 0.1, 0);

    set_translate(specchio, 3.5, 0.5, 0.4);

    /* place camera */
    set_translate(scene, -9.0, -9.0, -11.0);


    /* generate the scene */
    save_scene("tavolo.md", "Scene", scene);

    return 0;
  }
  else if(strcmp(argv[1], "teca") == 0) {

    /* load surfaces */
    box1 = read_nurbs("teca_box_1.db");
    box2 = read_nurbs("teca_box_2.db");
    box3 = read_nurbs("teca_box_3.db");
    box4 = read_nurbs("teca_box_4.db");
    box5 = read_nurbs("teca_box_5.db");
    box6 = read_nurbs("teca_box_6.db");
    asta_teca = read_nurbs("teca_asta.dbe");

    scabrote_testa = read_nurbs("scabrote_testa.dbe");
    scabrote_orecchie = read_nurbs("scabrote_orecchie.db");
    scabrote_zanna_sx = read_nurbs("scabrote_zanna_sx.db");
    scabrote_zanna_dx = read_nurbs("scabrote_zanna_dx.db");
    scabrote_corno = read_nurbs("scabrote_corno.dbe");
    scabrote_prot = read_nurbs("scabrote_protuberanze.db");

    box = create_list(box1, "Scatola");
    box = insert_in_list(box, box2);
    box = insert_in_list(box, box3);
    box = insert_in_list(box, box4);
    box = insert_in_list(box, box5);
    box = insert_in_list(box, box6);

    scabrote = create_list(scabrote_testa, "Scabrote");
    scabrote = insert_in_list(scabrote, scabrote_orecchie);
    scabrote = insert_in_list(scabrote, scabrote_zanna_sx);
    scabrote = insert_in_list(scabrote, scabrote_zanna_dx);
    scabrote = insert_in_list(scabrote, scabrote_corno);
    scabrote = insert_in_list(scabrote, scabrote_prot);

    teca = create_list(box, "Teca");
    teca = insert_in_list(teca, asta_teca);

    scene = create_list(teca, "Scene");

    stanza = create_list(pavimento, "Stanza");
    stanza = insert_in_list(stanza, soffitto);
    //stanza = insert_in_list(stanza, parete_dx);
    stanza = insert_in_list(stanza, parete_sx);
    stanza = insert_in_list(stanza, retro);

    scene = insert_in_list(scene, stanza);
    scene = insert_in_list(scene, scabrote);

    set_attribute(box, glass);
    set_attribute(asta_teca, darkGlass);
    set_bump_texture(scabrote, brown, "textures/fur.hr",
		     0.2, 0.8, 0.2, 0.8, 1, 0.2, 1.0);
    set_attribute(scabrote_zanna_sx, avorio);
    set_attribute(scabrote_zanna_dx, avorio);

    set_scale(stanza, 0.1, 0.1, 0.1);
    set_cube_texture(stanza, whiteCeramic, blackCeramic);
    set_scale(stanza, 1 / 0.1, 1 / 0.1, 1 / 0.1);

    /* place/scale objects */
    set_translate(box, 0.0, 0.0, 0.16);
    set_translate(teca, -4.0, 0.0, -1.0);
    set_translate(scabrote, -4.0, 0.0, -0.84);

    /* place camera */
    set_translate(scene, -7.0, -8.0, -3.0);
    set_z_rotate(scene, 180, 1);

    /* generate the scene */
    save_scene("teca.md", "Scene", scene);

    return 0;
  }
  else if(strcmp(argv[1], "stanza") == 0) {

    /* load surfaces */
    asse = read_nurbs("scale_asse.db");
    chiocciola= read_nurbs("scale_chiocciola.db");

    scene = create_list(asse, "Scene");
    scene = insert_in_list(scene, chiocciola);

    stanza = create_list(pavimento, "Stanza");
    stanza = insert_in_list(stanza, soffitto);
    //stanza = insert_in_list(stanza, parete_dx);
    stanza = insert_in_list(stanza, parete_sx);
    stanza = insert_in_list(stanza, retro);

    scene = insert_in_list(scene, stanza);

    set_attribute(asse, darkGlass);
    set_attribute(chiocciola, darkGlass);
    set_octant_texture(stanza, whiteCeramic, blackCeramic);

    /* place camera */
    set_translate(scene, 2.0, -8.0, -3.0);
    set_z_rotate(scene, -45, 1);

    /* generate the scene */
    save_scene("stanza.md", "Scene", scene);

    return 0;
  }
  else {
    fprintf(stderr, "Utilizzo : scenizza [tavolo | teca | stanza]\n");
    exit(-1);
  }

  return 0;
}
